@@ -1,6 +1,6 @@
 #' Fast Index Generation
 #' 
-#' For not too small m, much faster than `rep(seq_len(m), each = each)`.
+#' For not too small m much faster than `rep(seq_len(m), each = each)`.
 #' 
 #' @noRd
 #' @keywords internal
@@ -16,6 +16,14 @@ rep_each <- function(m, each) {
   dim(out) <- NULL
   out 
 }
+# 
+# # Same as rep.int(seq_len(m), times)
+# rep_times <- function(m, times) {
+#   out <- .row(dim = c(m, times))
+#   dim(out) <- NULL
+#   out
+# }
+
 
 #' Fast OHE
 #' 
@@ -50,10 +58,6 @@ fdummy <- function(x) {
 #' @param w Optional case weights.
 #' @returns A vector of column means.
 wcolMeans <- function(x, w = NULL) {
-  if (NCOL(x) == 1L && is.atomic(x) && !is.factor(x) && is.null(w)) {
-    # stat::weighted.mean() is much slower than via colSums()
-    return(mean(x))
-  }
   if (is.factor(x)) {
     if (is.null(w)) {
       return(colMeans_factor(x))
@@ -173,7 +177,8 @@ wrowmean_vector <- function(x, ngroups = 1L, w = NULL) {
   }
   nm <- if (is.matrix(x)) colnames(x)
   dim(x) <- c(length(x) %/% ngroups, ngroups)
-  out <- as.matrix(if (is.null(w)) colMeans(x) else colSums(x * w) / sum(w))
+  out <- if (is.null(w)) colMeans(x) else colSums(x * w) / sum(w)
+  dim(out) <- c(ngroups, 1L)
   if (!is.null(nm)) {
     colnames(out) <- nm
   }
@@ -266,3 +271,26 @@ wcenter <- function(x, w = NULL) {
   # sweep(x, MARGIN = 2L, STATS = wcolMeans(x, w = w))  # Slower
   x - matrix(wcolMeans(x, w = w), nrow = nrow(x), ncol = ncol(x), byrow = TRUE)
 }
+
+#' Fast Row Subsetting (from kernelshap)
+#' 
+#' Internal function used to row-subset data.frames.
+#' Brings a massive speed-up for data.frames. All other classes (tibble, data.table,
+#' matrix) are subsetted in the usual way.
+#' 
+#' @noRd
+#' @keywords internal
+#' 
+#' @param x A matrix-like object.
+#' @param i Logical or integer vector of rows to pick.
+#' @returns Subsetted version of `x`.
+rep_rows <- function(x, i) {
+  if (!(all(class(x) == "data.frame"))) {
+    return(x[i, , drop = FALSE])  # matrix, tibble, data.table, ...
+  }
+  # data.frame
+  out <- lapply(x, function(z) if (length(dim(z)) != 2L) z[i] else z[i, , drop = FALSE])
+  attr(out, "row.names") <- .set_row_names(length(i))
+  class(out) <- "data.frame"
+  out
+}
